@@ -1,5 +1,5 @@
 # orm/strategies.py
-# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -532,9 +532,11 @@ class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
         self.is_aliased_class = inspect(self.entity).is_aliased_class
 
         join_condition = self.parent_property._join_condition
-        self._lazywhere, self._bind_to_col, self._equated_columns = (
-            join_condition.create_lazy_clause()
-        )
+        (
+            self._lazywhere,
+            self._bind_to_col,
+            self._equated_columns,
+        ) = join_condition.create_lazy_clause()
 
         (
             self._rev_lazywhere,
@@ -679,7 +681,22 @@ class LazyLoader(AbstractRelationshipLoader, util.MemoizedSlots):
         ):
             return attributes.PASSIVE_NO_RESULT
 
-        if self._raise_always and not passive & attributes.NO_RAISE:
+        if (
+            # we were given lazy="raise"
+            self._raise_always
+            # the no_raise history-related flag was not passed
+            and not passive & attributes.NO_RAISE
+            and (
+                # if we are use_get and related_object_ok is disabled,
+                # which means we are at most looking in the identity map
+                # for history purposes or otherwise returning
+                # PASSIVE_NO_RESULT, don't raise.  This is also a
+                # history-related flag
+                not self.use_get
+                or passive & attributes.RELATED_OBJECT_OK
+            )
+        ):
+
             self._invoke_raise_load(state, passive, "raise")
 
         session = _state_session(state)
@@ -1005,11 +1022,11 @@ class SubqueryLoader(AbstractRelationshipLoader):
 
         # build up a path indicating the path from the leftmost
         # entity to the thing we're subquery loading.
-        with_poly_info = path.get(
+        with_poly_entity = path.get(
             context.attributes, "path_with_polymorphic", None
         )
-        if with_poly_info is not None:
-            effective_entity = with_poly_info.entity
+        if with_poly_entity is not None:
+            effective_entity = with_poly_entity
         else:
             effective_entity = self.entity
 
@@ -1501,11 +1518,13 @@ class JoinedLoader(AbstractRelationshipLoader):
                 chained_from_outerjoin,
             )
 
-        with_poly_info = path.get(
+        with_poly_entity = path.get(
             context.attributes, "path_with_polymorphic", None
         )
-        if with_poly_info is not None:
-            with_polymorphic = with_poly_info.with_polymorphic_mappers
+        if with_poly_entity is not None:
+            with_polymorphic = inspect(
+                with_poly_entity
+            ).with_polymorphic_mappers
         else:
             with_polymorphic = None
 
@@ -1523,7 +1542,7 @@ class JoinedLoader(AbstractRelationshipLoader):
             chained_from_outerjoin=chained_from_outerjoin,
         )
 
-        if with_poly_info is not None and None in set(
+        if with_poly_entity is not None and None in set(
             context.secondary_columns
         ):
             raise sa_exc.InvalidRequestError(
@@ -1552,7 +1571,6 @@ class JoinedLoader(AbstractRelationshipLoader):
 
         # otherwise figure it out.
         alias = loadopt.local_opts["eager_from_alias"]
-
         root_mapper, prop = path[-2:]
 
         if alias is not None:
@@ -1563,11 +1581,11 @@ class JoinedLoader(AbstractRelationshipLoader):
             )
         else:
             if path.contains(context.attributes, "path_with_polymorphic"):
-                with_poly_info = path.get(
+                with_poly_entity = path.get(
                     context.attributes, "path_with_polymorphic"
                 )
                 adapter = orm_util.ORMAdapter(
-                    with_poly_info.entity,
+                    with_poly_entity,
                     equivalents=prop.mapper._equivalent_columns,
                 )
             else:
@@ -1652,11 +1670,11 @@ class JoinedLoader(AbstractRelationshipLoader):
         parentmapper,
         chained_from_outerjoin,
     ):
-        with_poly_info = path.get(
+        with_poly_entity = path.get(
             context.attributes, "path_with_polymorphic", None
         )
-        if with_poly_info:
-            to_adapt = with_poly_info.entity
+        if with_poly_entity:
+            to_adapt = with_poly_entity
         else:
             to_adapt = self._gen_pooled_aliased_class(context)
 
@@ -1689,7 +1707,6 @@ class JoinedLoader(AbstractRelationshipLoader):
         context.create_eager_joins.append(
             (
                 self._create_eager_join,
-                context,
                 entity,
                 path,
                 adapter,
@@ -2207,12 +2224,12 @@ class SelectInLoader(AbstractRelationshipLoader, util.MemoizedSlots):
 
         # build up a path indicating the path from the leftmost
         # entity to the thing we're subquery loading.
-        with_poly_info = path_w_prop.get(
+        with_poly_entity = path_w_prop.get(
             context.attributes, "path_with_polymorphic", None
         )
 
-        if with_poly_info is not None:
-            effective_entity = with_poly_info.entity
+        if with_poly_entity is not None:
+            effective_entity = with_poly_entity
         else:
             effective_entity = self.entity
 
