@@ -1,5 +1,5 @@
 # sql/operators.py
-# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -153,8 +153,6 @@ class Operators(object):
          A value of 100 will be higher or equal to all operators, and -100
          will be lower than or equal to all operators.
 
-         .. versionadded:: 0.8 - added the 'precedence' argument.
-
         :param is_comparison: if True, the operator will be considered as a
          "comparison" operator, that is which evaluates to a boolean
          true/false value, like ``==``, ``>``, etc.  This flag should be set
@@ -322,15 +320,15 @@ class ColumnOperators(Operators):
     so that the ``==`` operation above is replaced by a clause
     construct.
 
-    See also:
+    .. seealso::
 
-    :ref:`types_operators`
+        :ref:`types_operators`
 
-    :attr:`.TypeEngine.comparator_factory`
+        :attr:`.TypeEngine.comparator_factory`
 
-    :class:`.ColumnOperators`
+        :class:`.ColumnOperators`
 
-    :class:`.PropComparator`
+        :class:`.PropComparator`
 
     """
 
@@ -530,6 +528,12 @@ class ColumnOperators(Operators):
 
             WHERE COL IN (?, ?, ?)
 
+        * A list of tuples may be provided if the comparison is against a
+          :func:`.tuple_` containing multiple expressions::
+
+            from sqlalchemy import tuple_
+            stmt.where(tuple_(col1, col2).in_([(1, 10), (2, 20), (3, 30)]))
+
         * An empty list, e.g.::
 
             stmt.where(column.in_([]))
@@ -568,9 +572,14 @@ class ColumnOperators(Operators):
 
           .. versionadded:: 1.2 added "expanding" bound parameters
 
-          The "expanding" feature in version 1.2 of SQLAlchemy does not
-          support passing an empty list as a parameter value; however,
-          version 1.3 does support this.
+          If an empty list is passed, a special "empty list" expression,
+          which is specific to the database in use, is rendered.  On
+          SQLite this would be::
+
+            WHERE COL IN (SELECT 1 FROM (SELECT 1) WHERE 1!=1)
+
+          .. versionadded:: 1.3 "expanding" bound parameters now support
+             empty lists
 
         * a :func:`.select` construct, which is usually a correlated
           scalar select::
@@ -624,8 +633,6 @@ class ColumnOperators(Operators):
         This is equivalent to using negation with
         :meth:`.ColumnOperators.like`, i.e. ``~x.like(y)``.
 
-        .. versionadded:: 0.8
-
         .. seealso::
 
             :meth:`.ColumnOperators.like`
@@ -639,8 +646,6 @@ class ColumnOperators(Operators):
         This is equivalent to using negation with
         :meth:`.ColumnOperators.ilike`, i.e. ``~x.ilike(y)``.
 
-        .. versionadded:: 0.8
-
         .. seealso::
 
             :meth:`.ColumnOperators.ilike`
@@ -656,8 +661,6 @@ class ColumnOperators(Operators):
         usage of ``IS`` may be desirable if comparing to boolean values
         on certain platforms.
 
-        .. versionadded:: 0.7.9
-
         .. seealso:: :meth:`.ColumnOperators.isnot`
 
         """
@@ -671,8 +674,6 @@ class ColumnOperators(Operators):
         usage of ``IS NOT`` may be desirable if comparing to boolean values
         on certain platforms.
 
-        .. versionadded:: 0.7.9
-
         .. seealso:: :meth:`.ColumnOperators.is_`
 
         """
@@ -695,7 +696,7 @@ class ColumnOperators(Operators):
         ``"%"`` and ``"_"`` that are present inside the <other> expression
         will behave like wildcards as well.   For literal string
         values, the :paramref:`.ColumnOperators.startswith.autoescape` flag
-        may be set to ``True`` to apply escaping to occurences of these
+        may be set to ``True`` to apply escaping to occurrences of these
         characters within the string value so that they match as themselves
         and not as wildcard characters.  Alternatively, the
         :paramref:`.ColumnOperators.startswith.escape` parameter will establish
@@ -783,7 +784,7 @@ class ColumnOperators(Operators):
         ``"%"`` and ``"_"`` that are present inside the <other> expression
         will behave like wildcards as well.   For literal string
         values, the :paramref:`.ColumnOperators.endswith.autoescape` flag
-        may be set to ``True`` to apply escaping to occurences of these
+        may be set to ``True`` to apply escaping to occurrences of these
         characters within the string value so that they match as themselves
         and not as wildcard characters.  Alternatively, the
         :paramref:`.ColumnOperators.endswith.escape` parameter will establish
@@ -871,7 +872,7 @@ class ColumnOperators(Operators):
         ``"%"`` and ``"_"`` that are present inside the <other> expression
         will behave like wildcards as well.   For literal string
         values, the :paramref:`.ColumnOperators.contains.autoescape` flag
-        may be set to ``True`` to apply escaping to occurences of these
+        may be set to ``True`` to apply escaping to occurrences of these
         characters within the string value so that they match as themselves
         and not as wildcard characters.  Alternatively, the
         :paramref:`.ColumnOperators.contains.escape` parameter will establish
@@ -955,7 +956,7 @@ class ColumnOperators(Operators):
         * Oracle - renders ``CONTAINS(x, y)``
         * other backends may provide special implementations.
         * Backends without any special implementation will emit
-          the operator as "MATCH".  This is compatible with SQlite, for
+          the operator as "MATCH".  This is compatible with SQLite, for
           example.
 
         """
@@ -1157,10 +1158,29 @@ class ColumnOperators(Operators):
         return self.reverse_operate(truediv, other)
 
 
+_commutative = {eq, ne, add, mul}
+_comparison = {eq, ne, lt, gt, ge, le}
+
+
+def commutative_op(fn):
+    _commutative.add(fn)
+    return fn
+
+
+def comparison_op(fn):
+    _comparison.add(fn)
+    return fn
+
+
 def from_():
     raise NotImplementedError()
 
 
+@comparison_op
+def function_as_comparison_op():
+    raise NotImplementedError()
+
+
 def as_():
     raise NotImplementedError()
 
@@ -1177,18 +1197,22 @@ def isfalse(a):
     raise NotImplementedError()
 
 
+@comparison_op
 def is_distinct_from(a, b):
     return a.is_distinct_from(b)
 
 
+@comparison_op
 def isnot_distinct_from(a, b):
     return a.isnot_distinct_from(b)
 
 
+@comparison_op
 def is_(a, b):
     return a.is_(b)
 
 
+@comparison_op
 def isnot(a, b):
     return a.isnot(b)
 
@@ -1201,34 +1225,42 @@ def op(a, opstring, b):
     return a.op(opstring)(b)
 
 
+@comparison_op
 def like_op(a, b, escape=None):
     return a.like(b, escape=escape)
 
 
+@comparison_op
 def notlike_op(a, b, escape=None):
     return a.notlike(b, escape=escape)
 
 
+@comparison_op
 def ilike_op(a, b, escape=None):
     return a.ilike(b, escape=escape)
 
 
+@comparison_op
 def notilike_op(a, b, escape=None):
     return a.notilike(b, escape=escape)
 
 
+@comparison_op
 def between_op(a, b, c, symmetric=False):
     return a.between(b, c, symmetric=symmetric)
 
 
+@comparison_op
 def notbetween_op(a, b, c, symmetric=False):
     return a.notbetween(b, c, symmetric=symmetric)
 
 
+@comparison_op
 def in_op(a, b):
     return a.in_(b)
 
 
+@comparison_op
 def notin_op(a, b):
     return a.notin_(b)
 
@@ -1265,34 +1297,42 @@ def _escaped_like_impl(fn, other, escape, autoescape):
     return fn(other, escape=escape)
 
 
+@comparison_op
 def startswith_op(a, b, escape=None, autoescape=False):
     return _escaped_like_impl(a.startswith, b, escape, autoescape)
 
 
+@comparison_op
 def notstartswith_op(a, b, escape=None, autoescape=False):
     return ~_escaped_like_impl(a.startswith, b, escape, autoescape)
 
 
+@comparison_op
 def endswith_op(a, b, escape=None, autoescape=False):
     return _escaped_like_impl(a.endswith, b, escape, autoescape)
 
 
+@comparison_op
 def notendswith_op(a, b, escape=None, autoescape=False):
     return ~_escaped_like_impl(a.endswith, b, escape, autoescape)
 
 
+@comparison_op
 def contains_op(a, b, escape=None, autoescape=False):
     return _escaped_like_impl(a.contains, b, escape, autoescape)
 
 
+@comparison_op
 def notcontains_op(a, b, escape=None, autoescape=False):
     return ~_escaped_like_impl(a.contains, b, escape, autoescape)
 
 
+@comparison_op
 def match_op(a, b, **kw):
     return a.match(b, **kw)
 
 
+@comparison_op
 def notmatch_op(a, b, **kw):
     return a.notmatch(b, **kw)
 
@@ -1301,14 +1341,20 @@ def comma_op(a, b):
     raise NotImplementedError()
 
 
+@comparison_op
 def empty_in_op(a, b):
     raise NotImplementedError()
 
 
+@comparison_op
 def empty_notin_op(a, b):
     raise NotImplementedError()
 
 
+def filter_op(a, b):
+    raise NotImplementedError()
+
+
 def concat_op(a, b):
     return a.concat(b)
 
@@ -1337,24 +1383,6 @@ def json_path_getitem_op(a, b):
     raise NotImplementedError()
 
 
-_commutative = {eq, ne, add, mul}
-
-_comparison = {
-    eq,
-    ne,
-    lt,
-    gt,
-    ge,
-    le,
-    between_op,
-    like_op,
-    is_,
-    isnot,
-    is_distinct_from,
-    isnot_distinct_from,
-}
-
-
 def is_comparison(op):
     return op in _comparison or isinstance(op, custom_op) and op.is_comparison
 
@@ -1411,6 +1439,7 @@ _largest = util.symbol("_largest", canonical=100)
 
 _PRECEDENCE = {
     from_: 15,
+    function_as_comparison_op: 15,
     any_op: 15,
     all_op: 15,
     getitem: 15,
@@ -1424,6 +1453,7 @@ _PRECEDENCE = {
     add: 7,
     sub: 7,
     concat_op: 6,
+    filter_op: 6,
     match_op: 5,
     notmatch_op: 5,
     ilike_op: 5,
